@@ -12,51 +12,97 @@ from ada.core.utils import Counter, create_guid, get_list_of_files, roundoff
 name_gen = Counter(1, "IfcEl")
 
 
-def create_ifcaxis2placement(ifcfile, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X):
+def ifc_p(f, p):
+    """
+
+    :param f:
+    :param p:
+    :type f: ifcopenshell.file.file
+    :return:
+    """
+    return f.create_entity("IfcCartesianPoint", to_real(p))
+
+
+def create_global_axes(f, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X):
     """
     Creates an IfcAxis2Placement3D from Location, Axis and RefDirection specified as Python tuples
 
-    :param ifcfile:
+    :param f:
     :param origin:
     :param loc_z:
     :param loc_x:
+    :type f: ifcopenshell.file.file
     :return:
     """
 
-    ifc_origin = ifcfile.createIfcCartesianPoint(to_real(origin))
-    ifc_loc_z = ifcfile.createIfcDirection(to_real(loc_z))
-    ifc_loc_x = ifcfile.createIfcDirection(to_real(loc_x))
-    axis2placement = ifcfile.createIfcAxis2Placement3D(ifc_origin, ifc_loc_z, ifc_loc_x)
+    ifc_origin = ifc_p(f, origin)
+    ifc_loc_z = f.createIfcDirection(to_real(loc_z))
+    ifc_loc_x = f.createIfcDirection(to_real(loc_x))
+    axis2placement = f.createIfcAxis2Placement3D(ifc_origin, ifc_loc_z, ifc_loc_x)
     return axis2placement
 
 
-def create_ifclocalplacement(ifcfile, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X, relative_to=None):
+def create_local_placement(f, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X, relative_to=None):
     """
     Creates an IfcLocalPlacement from Location, Axis and RefDirection,
     specified as Python tuples, and relative placement
 
-    :param ifcfile:
+    :param f:
     :param origin:
     :param loc_z:
     :param loc_x:
     :param relative_to:
+    :type f: ifcopenshell.file.file
     :return: IFC local placement
     """
 
-    axis2placement = create_ifcaxis2placement(ifcfile, origin, loc_z, loc_x)
-    ifclocalplacement2 = ifcfile.createIfcLocalPlacement(relative_to, axis2placement)
+    axis2placement = create_global_axes(f, origin, loc_z, loc_x)
+    ifclocalplacement2 = f.createIfcLocalPlacement(relative_to, axis2placement)
     return ifclocalplacement2
 
 
-def generate_tpl_ifc_file(file_name, project, organization, creator, schema, units):
+def create_new_ifc_file(file_name, schema):
+    import datetime
+
+    from .utils import get_version
+
+    f = ifcopenshell.file(schema=schema)
+    ada_ver = get_version()
+    f.wrapped_data.header.file_name.name = file_name
+    f.wrapped_data.header.file_name.time_stamp = (
+        datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc).astimezone().replace(microsecond=0).isoformat()
+    )
+
+    ver_str = f'IfcOpenShell: "{ifcopenshell.version}", ADA: "{ada_ver}"'
+
+    f.wrapped_data.header.file_name.preprocessor_version = ver_str
+    f.wrapped_data.header.file_name.originating_system = ver_str
+    f.wrapped_data.header.file_name.authorization = "Nobody"
+    length_unit = f.createIfcSIUnit(None, "LENGTHUNIT", None, "METRE")
+    f.createIfcUnitAssignment((length_unit,))
+    # f.wrapped_data.header.file_description.description = ("ViewDefinition[DesignTransferView]",)
+    return f
+
+
+def assembly_to_ifc_file(a):
+    """
+
+    :param a:
+    :type a: ada.Assembly
+    :return:
+    """
+    return generate_tpl_ifc_file(a.name, a.metadata["project"], a.metadata["schema"], a.units, a.user)
+
+
+def generate_tpl_ifc_file(file_name, project, schema, units, user):
     """
 
     :param file_name:
     :param project:
-    :param organization:
-    :param creator:
     :param schema:
     :param units:
+    :param user:
+    :type user: ada.config.User
     :return:
     """
 
@@ -77,8 +123,8 @@ def generate_tpl_ifc_file(file_name, project, organization, creator, schema, uni
     ifc_file = tpl_create(
         file_name + ".ifc",
         timestring,
-        organization,
-        creator,
+        user.org_name,
+        user.user_id,
         schema,
         application_version,
         int(timestamp),
@@ -86,6 +132,7 @@ def generate_tpl_ifc_file(file_name, project, organization, creator, schema, uni
         project_globalid,
         project,
         units_str,
+        user.org_name,
     )
 
     return ifc_file
@@ -292,7 +339,7 @@ def create_property_set(name, ifc_file, metadata_props):
 
 def add_properties_to_elem(name, ifc_file, ifc_elem, elem_props):
     """
-
+    :param name:
     :param ifc_file:
     :param ifc_elem:
     :param elem_props:
@@ -310,6 +357,15 @@ def add_properties_to_elem(name, ifc_file, ifc_elem, elem_props):
     )
 
 
+def add_multiple_props_to_elem(metadata_props, elem, f):
+    if len(metadata_props.keys()) > 0:
+        if type(list(metadata_props.values())[0]) is dict:
+            for pro_id, prop_ in metadata_props.items():
+                add_properties_to_elem(pro_id, f, elem, prop_)
+        else:
+            add_properties_to_elem("Properties", f, elem, metadata_props)
+
+
 def to_real(v):
     """
 
@@ -425,7 +481,10 @@ def get_representation(ifc_elem, settings):
 
     geom = get_geom(ifc_elem, settings)
     r, g, b, alpha = pdct_shape.styles[0]  # the shape color
-    return geom, (r, g, b), alpha
+
+    colour = None if (r, g, b) == (-1, -1, -1) else (r, g, b)
+
+    return geom, colour, alpha
 
 
 def get_geom(ifc_elem, settings):
@@ -524,8 +583,8 @@ def add_negative_extrusion(f, origin, loc_z, loc_x, depth, points, parent):
     owner_history = f.by_type("IfcOwnerHistory")[0]
 
     # Create and associate an opening for the window in the wall
-    opening_placement = create_ifclocalplacement(f, origin, loc_z, loc_x, parent.ObjectPlacement)
-    opening_axis_placement = create_ifcaxis2placement(f, origin, loc_z, loc_x)
+    opening_placement = create_local_placement(f, origin, loc_z, loc_x, parent.ObjectPlacement)
+    opening_axis_placement = create_global_axes(f, origin, loc_z, loc_x)
     polyline = create_ifcpolyline(f, points)
     ifcclosedprofile = f.createIfcArbitraryClosedProfileDef("AREA", None, polyline)
 
@@ -794,3 +853,49 @@ def convert_bm_jusl_to_ifc(bm):
         return 5
 
     return jusl_val
+
+
+def get_person(f, user_id):
+    for p in f.by_type("IfcPerson"):
+        if p.Identification == user_id:
+            return p
+    return None
+
+
+def get_org(f, org_id):
+    for p in f.by_type("IfcOrganization"):
+        if p.Identification == org_id:
+            return p
+    return None
+
+
+def create_reference_subrep(f, global_axes):
+    model_rep = f.create_entity("IfcGeometricRepresentationContext", None, "Model", 3, 1.0e-05, global_axes, None)
+    body_sub_rep = f.create_entity(
+        "IfcGeometricRepresentationSubContext",
+        "Body",
+        "Model",
+        None,
+        None,
+        None,
+        None,
+        model_rep,
+        None,
+        "MODEL_VIEW",
+        None,
+    )
+    ref_sub_rep = f.create_entity(
+        "IfcGeometricRepresentationSubContext",
+        "Reference",
+        "Model",
+        None,
+        None,
+        None,
+        None,
+        model_rep,
+        None,
+        "GRAPH_VIEW",
+        None,
+    )
+
+    return {"model": model_rep, "body": body_sub_rep, "reference": ref_sub_rep}
