@@ -35,7 +35,7 @@ __all__ = [
     "face_to_wires",
     "vector_length",
     "roundoff",
-    "parallel_check",
+    "is_parallel",
     "intersect_calc",
     "get_list_of_files",
     "is_occ_shape",
@@ -298,7 +298,18 @@ def distfunc(x, point, A, B):
     return vector_length(point - (A + x * AB))
 
 
-def parallel_check(ab, cd, tol=0.0001):
+def sort_points_by_dist(p, points):
+    return sorted(points, key=lambda x: vector_length(x - p))
+
+
+def is_point_on_line(a, b, p):
+    ap = p - a
+    ab = b - a
+    result = a + np.dot(ap, ab) / np.dot(ab, ab) * ab
+    return result
+
+
+def is_parallel(ab, cd, tol=0.0001):
     """
     Check if vectors AB and CD are parallel
 
@@ -376,7 +387,7 @@ def beam_cross_check(bm1, bm2, outofplane_tol=0.1):
     :type bm2: ada.Beam
     """
 
-    p_check = parallel_check
+    p_check = is_parallel
     i_check = intersect_calc
     v_len = vector_length
     a = bm1.n1.p
@@ -392,11 +403,6 @@ def beam_cross_check(bm1, bm2, outofplane_tol=0.1):
     ab_ = a + s * ab
     cd_ = c + t * cd
 
-    t_ = roundoff(t)
-    if 0 < t_ < 1:
-        logging.debug(f"Beam cross-check indicates that the beams {bm1} and {bm2} are most probably parallel")
-        return None
-
     if p_check(ab, cd):
         logging.debug(f"beams {bm1} {bm2} are parallel")
         return None
@@ -1539,11 +1545,11 @@ def normal_to_points_in_plane(points):
     v1 = p3 - p1
     v2 = p2 - p1
 
-    if parallel_check(v1, v2) is True:
+    if is_parallel(v1, v2) is True:
         for i in range(3, len(points)):
             p3 = points[i]
             v1 = p3 - p1
-            if parallel_check(v1, v2) is False:
+            if is_parallel(v1, v2) is False:
                 break
 
     # the cross product is a vector normal to the plane
@@ -3070,50 +3076,40 @@ def compute_minimal_distance_between_shapes(shp1, shp2):
     return dss
 
 
-def replace_node(fem, old, new):
+def replace_node(old_node, new_node):
     """
 
-    :param fem: ada.fem
-    :param old:
-    :param new:
-    :type fem: ada.fem.FEM
-    :type old: ada.Node
-    :type new: ada.Node
+    :param old_node:
+    :param new_node:
+    :type old_node: ada.Node
+    :type new_node: ada.Node
+    """
+    for elem in old_node.refs.copy():
+        node_index = elem.nodes.index(old_node)
+
+        elem.nodes.pop(node_index)
+        elem.nodes.insert(node_index, new_node)
+        elem.update()
+        # new_node.refs.extend(old_node.refs)
+        old_node.refs.pop(old_node.refs.index(elem))
+        new_node.refs.append(elem)
+        logging.debug(f"{old_node} exchanged with {new_node} --> {elem}")
+
+
+def replace_nodes_by_tol(nodes, decimals=0, tol=_Settings.point_tol):
     """
 
-    for elem in old.refs.copy():
-        # assert isinstance(ref, ada.fem.Elem)
-        try:
-            ref_index = elem.nodes.index(old)
-        except ValueError:
-            return
-        elem.nodes.pop(ref_index)
-
-        if new.id not in [e.id for e in elem.nodes]:
-            elem.nodes.insert(ref_index, new)
-
-        if len(elem.nodes) < 2:
-            fem.elements.remove_elements_by_id(elem.id)
-            old.refs.pop(old.refs.index(elem))
-            logging.error(f"Element removed due to coinciding nodes. Elem id {elem.id}")
-        elif len(elem.nodes) == (len(elem.edges_seq) - 1):
-            elem.change_type(len(elem.nodes))
-
-
-def replace_nodes_by_tol(fem, decimals=0, tol=1e-4):
-    """
-
-    :param fem:
+    :param nodes:
     :param decimals:
     :param tol:
-    :type fem: ada.fem.FEM
+    :type nodes: ada.core.containers.Nodes
     """
 
     def rounding(vec, decimals_):
         return np.around(vec, decimals=decimals_)
 
-    def n_is_most_precise(node, other_nodes_, decimals_=0):
-        most_precise = [np.array_equal(node.p, rounding(node.p, decimals_)) for node in [n] + other_nodes_]
+    def n_is_most_precise(n, nearby_nodes_, decimals_=0):
+        most_precise = [np.array_equal(n.p, rounding(n.p, decimals_)) for n in [node] + nearby_nodes_]
 
         if most_precise[0] and not np.all(most_precise[1:]):
             return True
@@ -3123,15 +3119,13 @@ def replace_nodes_by_tol(fem, decimals=0, tol=1e-4):
             logging.error(f"Recursion started at 0 decimals, but are now at {decimals_} decimals. Will proceed with n.")
             return True
         else:
-            return n_is_most_precise(node, other_nodes_, decimals_ + 1)
+            return n_is_most_precise(n, nearby_nodes_, decimals_ + 1)
 
-    for n in fem.nodes:
-        nearby_nodes = fem.nodes.get_by_volume(n.p, tol=tol)
-        other_nodes = list(filter(lambda x: x != n, nearby_nodes))
-        if len(other_nodes) > 0:
-            if n_is_most_precise(n, other_nodes, decimals):
-                for other_node in other_nodes:
-                    replace_node(fem, other_node, n)
+    for node in nodes:
+        nearby_nodes = list(filter(lambda x: x != node, nodes.get_by_volume(node.p, tol=tol)))
+        if nearby_nodes and n_is_most_precise(node, nearby_nodes, decimals):
+            for nearby_node in nearby_nodes:
+                replace_node(nearby_node, node)
 
 
 def calc_yvec(x_vec, z_vec=None):
@@ -3227,3 +3221,44 @@ def sweep_pipe(edge, xvec, r, wt):
     if boolean_result.IsNull():
         logging.debug("Boolean returns None")
     return boolean_result
+
+
+def make_name_fem_ready(value, no_dot=False):
+    """
+    Based on typically allowed names in FEM, this function will try to rename objects to comply without significant
+    changes to the original name
+
+    :param value:
+    :param no_dot:
+    :return: Fixed name
+    """
+    logging.debug("Converting bad name")
+    value = value.replace("/", "_").replace("=", "")
+    if str.isnumeric(value[0]):
+        value = "_" + value
+
+    if "/" in value:
+        logging.error(f'Character "/" found in {value}')
+
+    if no_dot:
+        value = value.replace(".", "_")
+
+    return value.strip()
+
+
+def get_version():
+    from importlib.metadata import version
+
+    return version("ada-py")
+
+
+def closest_val_in_dict(val, dct):
+    """
+    When mapping using a dictionary and value do not match with the keys in the dictionary.
+    :param val: Value a number, usually float
+    :param dct: Dictionary with number keys (int o float)
+    :return: Dictionary-value corresponding to the keys nearest the input value
+    """
+    table_looksups = np.array(list(dct))
+    dct_index = table_looksups[np.abs(table_looksups - val).argmin()]
+    return dct[dct_index]
