@@ -2,6 +2,7 @@ import logging
 
 import numpy as np
 
+from ..config import Settings as _Settings
 from . import constants as co
 from .containers import FemElements, FemSections, FemSets
 
@@ -54,9 +55,15 @@ class FemBase:
 
     @name.setter
     def name(self, value):
+        from ada.core.utils import make_name_fem_ready
+
         if str.isnumeric(value[0]):
             raise ValueError("Name cannot start with numeric")
-        self._name = value
+
+        if _Settings.convert_bad_names_for_fem:
+            self._name = make_name_fem_ready(value)
+        else:
+            self._name = value.strip()
 
     @property
     def parent(self):
@@ -180,16 +187,10 @@ class ElemShapes:
             raise ValueError(f'element type "{el_name}" is not yet supported')
 
     def __init__(self, el_type, nodes):
-        self.type = el_type.upper()
-        if ElemShapes.is_valid_elem(el_type) is False:
-            raise ValueError(f'Currently unsupported element type "{el_type}".')
-
-        num_nodes = ElemShapes.num_nodes(self.type)
-        if len(nodes) != num_nodes:
-            raise ValueError(f'Number of passed nodes "{len(nodes)}" does not match expected "{num_nodes}" ')
-
-        self.nodes = nodes
+        self.type = None
+        self.nodes = None
         self._edges = None
+        self.update(el_type, nodes)
 
     @property
     def edges(self):
@@ -208,6 +209,20 @@ class ElemShapes:
         else:
             return self._edges
 
+    def update(self, el_type=None, nodes=None):
+        if el_type is not None:
+            self.type = el_type.upper()
+            if ElemShapes.is_valid_elem(el_type) is False:
+                raise ValueError(f'Currently unsupported element type "{el_type}".')
+
+        nodes = self.nodes if nodes is None else nodes
+        num_nodes = ElemShapes.num_nodes(self.type)
+        if len(nodes) != num_nodes:
+            raise ValueError(f'Number of passed nodes "{len(nodes)}" does not match expected "{num_nodes}" ')
+
+        self.nodes = nodes
+        self._edges = None
+
     @property
     def faces(self):
         if self.type.upper() in ElemShapes.volume:
@@ -459,7 +474,7 @@ v
             #     (0,8), (8,1), (3,13), (13,2)
             # ]
         else:
-            print("Element type {} is currently not supported".format(self.type))
+            logging.error(f"Element type {self.type} is currently not supported")
 
     @property
     def _cube_faces(self):
@@ -485,6 +500,9 @@ v
         else:
             logging.error(f"Element type {self.type} is currently not supported for visualization")
 
+    def __repr__(self):
+        return f'{self.__class__.__name__}(Type: {self.type}, NodeIds: "{self.nodes}")'
+
 
 class FEM(FemBase):
     """
@@ -786,7 +804,7 @@ class FEM(FemBase):
         from ada.core.utils import vector_length
 
         edited_nodes = dict()
-        tol = 1e-3
+        tol = _Settings.point_tol
 
         def build_mpc(fs):
             """
@@ -868,6 +886,10 @@ class FEM(FemBase):
                 elem.parent.nodes.add(n2, allow_coincident=True)
                 i = elem.nodes.index(n)
                 elem.nodes[i] = n2
+                if elem.fem_sec.offset is not None:
+                    if n in [x[0] for x in elem.fem_sec.offset]:
+                        elem.fem_sec.offset[i] = (n2, elem.fem_sec.offset[i][1])
+
                 s_set = FemSet(f"el{elem.id}_hinge{i + 1}_s", [n], "nset")
                 m_set = FemSet(f"el{elem.id}_hinge{i + 1}_m", [n2], "nset")
                 elem.parent.add_set(m_set)
@@ -1448,7 +1470,7 @@ class Elem(FemBase):
         self.type = el_type.upper()
         self._el_id = el_id
 
-        self._shape = ElemShapes(self.type, nodes)
+        self._shape = None
 
         if type(nodes[0]) is Node:
             for node in nodes:
@@ -1493,7 +1515,7 @@ class Elem(FemBase):
     @id.setter
     def id(self, value):
         if type(value) not in (np.int32, int, np.uint64) and issubclass(type(self), Connector) is False:
-            raise ValueError(f'Element name type "{type(value)}" must be numeric')
+            raise ValueError(f'Element ID "{type(value)}" must be numeric')
         self._el_id = value
 
     @property
@@ -1537,12 +1559,23 @@ class Elem(FemBase):
         :return:
         :rtype: ada.fem.ElemShapes
         """
+        if self._shape is None:
+            self._shape = ElemShapes(self.type, self.nodes)
         return self._shape
 
     @property
     def refs(self):
         return self._refs
 
+    def update(self):
+        from toolz import unique
+
+        self._nodes = list(unique(self.nodes))
+        if len(self.nodes) <= 1:
+            self._el_id = None
+        else:
+            self._shape = None
+
     def __repr__(self):
         return f'Elem(ID: {self._el_id}, Type: {self.type}, NodeIds: "{self.nodes}")'
 
@@ -2432,7 +2465,7 @@ class Step(FemBase):
     :param stabilize: Default=None.
     :param dyn_type: Dynamic analysis type 'TRANSIENT FIDELITY' | 'QUASI-STATIC'
     :param init_accel_calc: Initial acceleration calculation
-    :param eigenmodes: Eigenmodes
+    :param eigenmodes: Number of requested Eigenmodes
     :param alpha: Rayleigh Damping for use in Steady State analysis
     :param beta: Rayleigh Damping for use in Steady State analysis
     :param nodeid: Node ID for use in Steady State analysis
