@@ -1,3 +1,5 @@
+import logging
+
 from ada.config import Settings
 from ada.fem.io.utils import get_exe_path
 
@@ -99,7 +101,12 @@ class CodeAsterAnalysis(LocalExecute):
 
     def run(self, exit_on_complete=True):
         if Settings.use_docker_execute is False:
-            exe_path = get_exe_path("code_aster")
+            try:
+                exe_path = get_exe_path("code_aster")
+            except FileNotFoundError as e:
+                logging.error(e)
+                return
+
             self._run_local(f'"{exe_path}" {self.analysis_name}.export', exit_on_complete=exit_on_complete)
         else:
             self._run_docker()
