@@ -139,7 +139,7 @@ USER:     {user}            ACCOUNT:     \n"""
 
         :return:
         """
-        from ada.core.utils import Counter
+        from ada.core.utils import Counter, make_name_fem_ready
         from ada.sections import SectionCat
 
         sec_str = ""
@@ -158,12 +158,13 @@ USER:     {user}            ACCOUNT:     \n"""
                 section = fem_sec.section
                 if section not in sec_ids:
                     secid = next(bmid)
+                    sec_name = make_name_fem_ready(fem_sec.section.name, no_dot=True)
                     section.metadata["numid"] = secid
                     names_str += self.write_ff(
                         "TDSECT",
                         [
-                            (4, secid, 100 + len(fem_sec.section.name), 0),
-                            (fem_sec.section.name,),
+                            (4, secid, 100 + len(sec_name), 0),
+                            (sec_name,),
                         ],
                     )
                     sec_ids.append(fem_sec.section)
@@ -172,9 +173,10 @@ USER:     {user}            ACCOUNT:     \n"""
                         beam = fem_sec.metadata["beam"]
                         numel = fem_sec.metadata["numel"]
                         fem_sec.metadata["ircon"] = next(ircon)
+                        bm_name = make_name_fem_ready(beam.name, no_dot=True)
                         concept_str += self.write_ff(
                             "TDSCONC",
-                            [(4, fem_sec.metadata["ircon"], 100 + len(beam.guid), 0), (beam.guid,)],
+                            [(4, fem_sec.metadata["ircon"], 100 + len(bm_name), 0), (bm_name,)],
                         )
                         concept_str += self.write_ff("SCONCEPT", [(8, next(c), 7, 0), (0, 1, 0, 2)])
                         sconc_ref = next(c)
@@ -234,10 +236,15 @@ USER:     {user}            ACCOUNT:     \n"""
                                 (section.t_ftop, section.w_btn, p.Sfy, p.Sfz),
                             ],
                         )
+                    elif SectionCat.is_tubular_profile(section.type):
+                        sec_str += self.write_ff(
+                            "GPIPE",
+                            [(secid, (section.r - section.wt) * 2, section.r * 2, section.wt), (p.Sfy, p.Sfz)],
+                        )
                     elif SectionCat.is_circular_profile(section.type):
                         sec_str += self.write_ff(
                             "GPIPE",
-                            [(secid, section.r - section.wt, section.r, section.wt), (p.Sfy, p.Sfz)],
+                            [(secid, (section.r - section.r * 0.99) * 2, section.r * 2, section.wt), (p.Sfy, p.Sfz)],
                         )
                     elif SectionCat.is_flatbar(section.type):
                         sec_str += self.write_ff(
@@ -269,7 +276,7 @@ USER:     {user}            ACCOUNT:     \n"""
 
         :return:
         """
-        from .reader import SesamReader
+        from .reader import eltype_2_sesam
 
         elements = self._gelements
 
@@ -278,7 +285,7 @@ USER:     {user}            ACCOUNT:     \n"""
                 self.write_ff(
                     "GELMNT1",
                     [
-                        (el.id, el.id, SesamReader.eltype_2_sesam(el.type), 0),
+                        (el.id, el.id, eltype_2_sesam(el.type), 0),
                         ([n.id for n in el.nodes]),
                     ],
                 )
